from enum import Enum


class ModelTypeEnum(str, Enum):
    TEXT2IMAGE = "text2image"
    TEXT2TEXT = "text2text"
    TEXT2VECTOR = "text2vector"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.role_enum import RoleEnum

T = TypeVar("T", bound="Message")


class MessageDict(TypedDict):
    role: RoleEnum
    content: str
    pass


@_attrs_define
class Message:
    """
    Attributes:
        role (RoleEnum): * `user` - user
            * `assistant` - assistant
        content (str): The content of the message.
    """

    role: RoleEnum
    content: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        role = self.role.value

        content = self.content

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
                "content": content,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        role = RoleEnum(d.pop("role"))

        content = d.pop("content")

        message = cls(
            role=role,
            content=content,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.document_type_enum import DocumentTypeEnum
from ..models.status_enum import StatusEnum

T = TypeVar("T", bound="DocumentOutput")


class DocumentOutputDict(TypedDict):
    repository_id: int
    document_id: int
    name: str
    document_type: DocumentTypeEnum
    status: StatusEnum
    error: Union[None, str]
    chunk_count: int = 0
    pass


@_attrs_define
class DocumentOutput:
    """
    Attributes:
        repository_id (int):
        document_id (int):
        name (str):
        document_type (DocumentTypeEnum): * `text` - text
        status (StatusEnum): * `PENDING` - Pending
            * `UPLOADED` - Uploaded
            * `PARSING` - Parsing
            * `CHUNKING` - Chunking
            * `WAITING_FOR_CHUNKS_COMPLETION` - Waiting for chunks completion
            * `PROCESSING` - Processing
            * `COMPLETED` - Completed
            * `FAILED` - Failed
        error (Union[None, str]):
        chunk_count (int):  Default: 0.
    """

    repository_id: int
    document_id: int
    name: str
    document_type: DocumentTypeEnum
    status: StatusEnum
    error: Union[None, str]
    chunk_count: int = 0

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        repository_id = self.repository_id

        document_id = self.document_id

        name = self.name

        document_type = self.document_type.value

        status = self.status.value

        error: Union[None, str]
        error = self.error

        chunk_count = self.chunk_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "repository_id": repository_id,
                "document_id": document_id,
                "name": name,
                "document_type": document_type,
                "status": status,
                "error": error,
                "chunk_count": chunk_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        repository_id = d.pop("repository_id")

        document_id = d.pop("document_id")

        name = d.pop("name")

        document_type = DocumentTypeEnum(d.pop("document_type"))

        status = StatusEnum(d.pop("status"))

        def _parse_error(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        error = _parse_error(d.pop("error"))

        chunk_count = d.pop("chunk_count")

        document_output = cls(
            repository_id=repository_id,
            document_id=document_id,
            name=name,
            document_type=document_type,
            status=status,
            error=error,
            chunk_count=chunk_count,
        )

        document_output.additional_properties = d
        return document_output

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.embedding import Embedding
from ..models.usage import Usage

T = TypeVar("T", bound="EmbeddingsResponse")


class EmbeddingsResponseDict(TypedDict):
    data: List["Embedding"]
    model: str
    usage: "Usage"
    provider_name: str
    provider_id: str
    pass


@_attrs_define
class EmbeddingsResponse:
    """
    Attributes:
        data (List['Embedding']): The embeddings for the input.
        model (str): The model to generate the embeddings.
        usage (Usage):
        provider_name (str): The name of the provider that generated the completion.
        provider_id (str): The ID of the provider that generated the completion.
    """

    data: List["Embedding"]
    model: str
    usage: "Usage"
    provider_name: str
    provider_id: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        model = self.model

        usage = self.usage.to_dict()

        provider_name = self.provider_name

        provider_id = self.provider_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
                "model": model,
                "usage": usage,
                "provider_name": provider_name,
                "provider_id": provider_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.embedding import Embedding
        from ..models.usage import Usage

        d = src_dict.copy() if src_dict else {}
        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = Embedding.from_dict(data_item_data)

            data.append(data_item)

        model = d.pop("model")

        usage = Usage.from_dict(d.pop("usage"))

        provider_name = d.pop("provider_name")

        provider_id = d.pop("provider_id")

        embeddings_response = cls(
            data=data,
            model=model,
            usage=usage,
            provider_name=provider_name,
            provider_id=provider_id,
        )

        embeddings_response.additional_properties = d
        return embeddings_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains methods for accessing the API """


from typing_extensions import Unpack

from ..models import (
    ChatCompletionInputDict,
    DocumentInputDict,
    EmbeddingsInputDict,
    FineTuningInputDict,
    InputDataPointDict,
    PatchedDataPointDict,
)
from .chat_completions.v1_chat_completions_create import v1_chat_completions_create_wrapper
from .datapoints.v1_data_points_create import v1_data_points_create_wrapper
from .datapoints.v1_data_points_destroy import v1_data_points_destroy_wrapper
from .datapoints.v1_data_points_list import v1_data_points_list_wrapper
from .datapoints.v1_data_points_partial_update import v1_data_points_partial_update_wrapper
from .datapoints.v1_data_points_retrieve import v1_data_points_retrieve_wrapper
from .datapoints.v1_data_points_update import v1_data_points_update_wrapper
from .embeddings.v1_embeddings_create import v1_embeddings_create_wrapper
from .finetuning.v1_finetuning_create import v1_finetuning_create_wrapper
from .finetuning.v1_finetuning_retrieve import v1_finetuning_retrieve_wrapper
from .models.v1_models_list import v1_models_list_wrapper
from .models.v1_models_retrieve import v1_models_retrieve_wrapper
from .repository_document.v1_repository_document_create import v1_repository_document_create_wrapper


class ChatCompletionsModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[ChatCompletionInputDict]):
        return v1_chat_completions_create_wrapper(self._client)(**kwargs)


class DatapointsModule:
    def __init__(self, client):
        self._client = client

    def list(
        self,
    ):
        return v1_data_points_list_wrapper(self._client)()

    def create(self, **kwargs: Unpack[InputDataPointDict]):
        return v1_data_points_create_wrapper(self._client)(**kwargs)

    def retrieve(
        self,
        id: int,
    ):
        return v1_data_points_retrieve_wrapper(self._client)(
            id,
        )

    def update(self, id: int, **kwargs: Unpack[InputDataPointDict]):
        return v1_data_points_update_wrapper(self._client)(id, **kwargs)

    def delete(
        self,
        id: int,
    ):
        return v1_data_points_destroy_wrapper(self._client)(
            id,
        )

    def patch(self, id: int, **kwargs: Unpack[PatchedDataPointDict]):
        return v1_data_points_partial_update_wrapper(self._client)(id, **kwargs)


class EmbeddingsModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[EmbeddingsInputDict]):
        return v1_embeddings_create_wrapper(self._client)(**kwargs)


class FinetuningModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[FineTuningInputDict]):
        return v1_finetuning_create_wrapper(self._client)(**kwargs)

    def retrieve(
        self,
        job_id: str,
    ):
        return v1_finetuning_retrieve_wrapper(self._client)(
            job_id,
        )


class ModelsModule:
    def __init__(self, client):
        self._client = client

    def list(
        self,
    ):
        return v1_models_list_wrapper(self._client)()

    def retrieve(
        self,
        id: int,
    ):
        return v1_models_retrieve_wrapper(self._client)(
            id,
        )


class RepositoryDocumentModule:
    def __init__(self, client):
        self._client = client

    def create(self, repository_id: int, **kwargs: Unpack[DocumentInputDict]):
        return v1_repository_document_create_wrapper(self._client)(repository_id, **kwargs)


class ChatModuleWrapper:
    completions: ChatCompletionsModule

    def __init__(self, client):
        self.completions = ChatCompletionsModule(client)


class RepositoryModuleWrapper:
    document: RepositoryDocumentModule

    def __init__(self, client):
        self.document = RepositoryDocumentModule(client)



""" Contains all the data models used in inputs/outputs """

from .api_response_validation_error import APIResponseValidationError
from .api_response_validation_error_code_enum import APIResponseValidationErrorCodeEnum
from .authentication_error import AuthenticationError
from .authentication_error_code_enum import AuthenticationErrorCodeEnum
from .blank_enum import BlankEnum
from .catch_all_error import CatchAllError
from .catch_all_error_code_enum import CatchAllErrorCodeEnum
from .chat_completion_input import ChatCompletionInput, ChatCompletionInputDict
from .chat_completion_input_logit_bias_type_0 import ChatCompletionInputLogitBiasType0
from .chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0
from .chat_completion_input_tools_item import ChatCompletionInputToolsItem
from .chat_completion_response import ChatCompletionResponse
from .conflict_error import ConflictError
from .conflict_error_code_enum import ConflictErrorCodeEnum
from .data_point import DataPoint
from .document_chunks import DocumentChunks
from .document_input import DocumentInput, DocumentInputDict
from .document_output import DocumentOutput
from .document_type_enum import DocumentTypeEnum
from .embedding import Embedding
from .embeddings_input import EmbeddingsInput, EmbeddingsInputDict
from .embeddings_response import EmbeddingsResponse
from .encoding_format_enum import EncodingFormatEnum
from .enhancement import Enhancement
from .fine_tuning_input import FineTuningInput, FineTuningInputDict
from .fine_tuning_response import FineTuningResponse
from .fine_tuning_sample import FineTuningSample
from .input_data_point import InputDataPoint, InputDataPointDict
from .message import Message
from .model import Model
from .model_not_found_error import ModelNotFoundError
from .model_not_found_error_code_enum import ModelNotFoundErrorCodeEnum
from .model_provider_enum import ModelProviderEnum
from .model_type_enum import ModelTypeEnum
from .patched_data_point import PatchedDataPoint, PatchedDataPointDict
from .permission_denied_error import PermissionDeniedError
from .permission_denied_error_code_enum import PermissionDeniedErrorCodeEnum
from .provider_api_connection_error import ProviderAPIConnectionError
from .provider_api_connection_error_code_enum import ProviderAPIConnectionErrorCodeEnum
from .provider_api_status_error import ProviderAPIStatusError
from .provider_api_status_error_code_enum import ProviderAPIStatusErrorCodeEnum
from .provider_api_timeout_error import ProviderAPITimeoutError
from .provider_api_timeout_error_code_enum import ProviderAPITimeoutErrorCodeEnum
from .provider_internal_server_error import ProviderInternalServerError
from .provider_internal_server_error_code_enum import ProviderInternalServerErrorCodeEnum
from .provider_not_found_error import ProviderNotFoundError
from .provider_not_found_error_code_enum import ProviderNotFoundErrorCodeEnum
from .rate_limit_error import RateLimitError
from .rate_limit_error_code_enum import RateLimitErrorCodeEnum
from .response_choice import ResponseChoice
from .retrieve_fine_tuning_response import RetrieveFineTuningResponse
from .role_enum import RoleEnum
from .status_enum import StatusEnum
from .unprocessable_entity_error import UnprocessableEntityError
from .unprocessable_entity_error_code_enum import UnprocessableEntityErrorCodeEnum
from .usage import Usage
from .validation_detail import ValidationDetail
from .validation_detail_error_messages_item import ValidationDetailErrorMessagesItem
from .validation_error import ValidationError
from .validation_error_code_enum import ValidationErrorCodeEnum
from .validation_error_details import ValidationErrorDetails

__all__ = (
    "APIResponseValidationError",
    "APIResponseValidationErrorCodeEnum",
    "AuthenticationError",
    "AuthenticationErrorCodeEnum",
    "BlankEnum",
    "CatchAllError",
    "CatchAllErrorCodeEnum",
    "ChatCompletionInput",
    "ChatCompletionInputLogitBiasType0",
    "ChatCompletionInputResponseFormatType0",
    "ChatCompletionInputToolsItem",
    "ChatCompletionResponse",
    "ConflictError",
    "ConflictErrorCodeEnum",
    "DataPoint",
    "DocumentChunks",
    "DocumentInput",
    "DocumentOutput",
    "DocumentTypeEnum",
    "Embedding",
    "EmbeddingsInput",
    "EmbeddingsResponse",
    "EncodingFormatEnum",
    "Enhancement",
    "FineTuningInput",
    "FineTuningResponse",
    "FineTuningSample",
    "InputDataPoint",
    "Message",
    "Model",
    "ModelNotFoundError",
    "ModelNotFoundErrorCodeEnum",
    "ModelProviderEnum",
    "ModelTypeEnum",
    "PatchedDataPoint",
    "PermissionDeniedError",
    "PermissionDeniedErrorCodeEnum",
    "ProviderAPIConnectionError",
    "ProviderAPIConnectionErrorCodeEnum",
    "ProviderAPIStatusError",
    "ProviderAPIStatusErrorCodeEnum",
    "ProviderAPITimeoutError",
    "ProviderAPITimeoutErrorCodeEnum",
    "ProviderInternalServerError",
    "ProviderInternalServerErrorCodeEnum",
    "ProviderNotFoundError",
    "ProviderNotFoundErrorCodeEnum",
    "RateLimitError",
    "RateLimitErrorCodeEnum",
    "ResponseChoice",
    "RetrieveFineTuningResponse",
    "RoleEnum",
    "StatusEnum",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorCodeEnum",
    "Usage",
    "ValidationDetail",
    "ValidationDetailErrorMessagesItem",
    "ValidationError",
    "ValidationErrorCodeEnum",
    "ValidationErrorDetails",
    "ChatCompletionInputDict",
    "ChatCompletionInputDict",
    "ChatCompletionInputDict",
    "InputDataPointDict",
    "InputDataPointDict",
    "InputDataPointDict",
    "InputDataPointDict",
    "InputDataPointDict",
    "InputDataPointDict",
    "PatchedDataPointDict",
    "PatchedDataPointDict",
    "PatchedDataPointDict",
    "EmbeddingsInputDict",
    "EmbeddingsInputDict",
    "EmbeddingsInputDict",
    "FineTuningInputDict",
    "FineTuningInputDict",
    "FineTuningInputDict",
    "DocumentInputDict",
    "DocumentInputDict",
    "DocumentInputDict",
)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="DocumentChunks")


class DocumentChunksDict(TypedDict):
    repository_id: NotRequired[Union[Unset, int]]
    document_id: NotRequired[Union[Unset, int]]
    chunk_id: NotRequired[Union[Unset, int]]
    document_name: NotRequired[Union[Unset, str]]
    similarity_score: NotRequired[Union[Unset, float]]
    content: NotRequired[Union[Unset, str]]
    pass


@_attrs_define
class DocumentChunks:
    """
    Attributes:
        repository_id (Union[Unset, int]):
        document_id (Union[Unset, int]):
        chunk_id (Union[Unset, int]):
        document_name (Union[Unset, str]):
        similarity_score (Union[Unset, float]):
        content (Union[Unset, str]):
    """

    repository_id: Union[Unset, int] = UNSET
    document_id: Union[Unset, int] = UNSET
    chunk_id: Union[Unset, int] = UNSET
    document_name: Union[Unset, str] = UNSET
    similarity_score: Union[Unset, float] = UNSET
    content: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        repository_id = self.repository_id

        document_id = self.document_id

        chunk_id = self.chunk_id

        document_name = self.document_name

        similarity_score = self.similarity_score

        content = self.content

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if repository_id is not UNSET:
            field_dict["repository_id"] = repository_id
        if document_id is not UNSET:
            field_dict["document_id"] = document_id
        if chunk_id is not UNSET:
            field_dict["chunk_id"] = chunk_id
        if document_name is not UNSET:
            field_dict["document_name"] = document_name
        if similarity_score is not UNSET:
            field_dict["similarity_score"] = similarity_score
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        repository_id = d.pop("repository_id", UNSET)

        document_id = d.pop("document_id", UNSET)

        chunk_id = d.pop("chunk_id", UNSET)

        document_name = d.pop("document_name", UNSET)

        similarity_score = d.pop("similarity_score", UNSET)

        content = d.pop("content", UNSET)

        document_chunks = cls(
            repository_id=repository_id,
            document_id=document_id,
            chunk_id=chunk_id,
            document_name=document_name,
            similarity_score=similarity_score,
            content=content,
        )

        document_chunks.additional_properties = d
        return document_chunks

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ModelProviderEnum(str, Enum):
    ANTHROPIC = "anthropic"
    ANYSCALE = "anyscale"
    AZURE = "azure"
    CLOUDFLARE = "cloudflare"
    COHERE = "cohere"
    DEEPINFRA = "deepinfra"
    FIREWORKSAI = "fireworksai"
    GROQ = "groq"
    LAMINI = "lamini"
    MISTRALAI = "mistralai"
    OCTOAI = "octoai"
    OPENAI = "openai"
    OPENROUTER = "openrouter"
    PERPLEXITY = "perplexity"
    PREM = "prem"
    REPLICATE = "replicate"
    TOGETHER = "together"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Dict, Optional, Union

import httpx
from typing_extensions import Any, Unpack

from ... import errors
from ...models.api_response_validation_error import APIResponseValidationError
from ...models.authentication_error import AuthenticationError
from ...models.catch_all_error import CatchAllError
from ...models.conflict_error import ConflictError
from ...models.fine_tuning_input import FineTuningInput
from ...models.fine_tuning_response import FineTuningResponse
from ...models.model_not_found_error import ModelNotFoundError
from ...models.permission_denied_error import PermissionDeniedError
from ...models.provider_api_connection_error import ProviderAPIConnectionError
from ...models.provider_api_status_error import ProviderAPIStatusError
from ...models.provider_api_timeout_error import ProviderAPITimeoutError
from ...models.provider_internal_server_error import ProviderInternalServerError
from ...models.provider_not_found_error import ProviderNotFoundError
from ...models.rate_limit_error import RateLimitError
from ...models.unprocessable_entity_error import UnprocessableEntityError
from ...models.validation_error import ValidationError

# from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    **body: Unpack[FineTuningInput],
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/v1/finetuning",
    }

    _json_body = body

    _kwargs["json"] = _json_body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client, response: httpx.Response
) -> Optional[
    Union[
        AuthenticationError,
        ConflictError,
        FineTuningResponse,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = FineTuningResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client, response: httpx.Response
) -> Response[
    Union[
        AuthenticationError,
        ConflictError,
        FineTuningResponse,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_finetuning_create_wrapper(client):
    def v1_finetuning_create_wrapped(
        **body: Unpack[FineTuningInput],
    ) -> Union[
        AuthenticationError,
        ConflictError,
        FineTuningResponse,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]:
        """Creates a finetuning job.

        Args:
            body (FineTuningInput):
            body (FineTuningInput):
            body (FineTuningInput):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[Union[AuthenticationError, ConflictError, FineTuningResponse, PermissionDeniedError, RateLimitError, Union['APIResponseValidationError', 'CatchAllError', 'ProviderAPIConnectionError', 'ProviderAPIStatusError', 'ProviderAPITimeoutError', 'ProviderInternalServerError'], Union['ModelNotFoundError', 'ProviderNotFoundError'], UnprocessableEntityError, ValidationError]]
        """

        kwargs = _get_kwargs(
            **body,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return v1_finetuning_create_wrapped



import json
from typing import Dict, List, Tuple, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.chat_completion_input_logit_bias_type_0 import ChatCompletionInputLogitBiasType0
from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0
from ..models.chat_completion_input_tools_item import ChatCompletionInputToolsItem
from ..models.enhancement import Enhancement
from ..models.message import Message
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatCompletionInput")


class ChatCompletionInputDict(TypedDict):
    project_id: int
    messages: List["Message"]
    session_id: NotRequired[Union[Unset, str]]
    repositories: NotRequired[Union[Unset, Enhancement]]
    model: NotRequired[Union[Unset, str]]
    system_prompt: NotRequired[Union[Unset, str]]
    frequency_penalty: NotRequired[Union[Unset, float]]
    logit_bias: NotRequired[Union["ChatCompletionInputLogitBiasType0", None, Unset]]
    max_tokens: NotRequired[Union[None, Unset, int]]
    presence_penalty: NotRequired[Union[Unset, float]]
    response_format: NotRequired[Union["ChatCompletionInputResponseFormatType0", None, Unset]]
    seed: NotRequired[Union[None, Unset, int]]
    stop: NotRequired[Union[None, Unset, str]]
    stream: NotRequired[Union[Unset, bool]]
    temperature: NotRequired[Union[None, Unset, float]]
    top_p: NotRequired[Union[None, Unset, float]]
    tools: NotRequired[Union[Unset, List["ChatCompletionInputToolsItem"]]]
    user: NotRequired[Union[None, Unset, str]]
    pass


@_attrs_define
class ChatCompletionInput:
    """
    Attributes:
        project_id (int): The ID of the project to use.
        messages (List['Message']): A list of messages comprising the conversation so far.
        session_id (Union[Unset, str]): The ID of the session to use. It helps to track the chat history.
        repositories (Union[Unset, Enhancement]):
        model (Union[Unset, str]): ID of the model to use. See the model endpoint compatibility table for details.
        system_prompt (Union[Unset, str]): The system prompt to use.
        frequency_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on their existing frequency.
        logit_bias (Union['ChatCompletionInputLogitBiasType0', None, Unset]): JSON object that maps tokens to an
            associated bias value from -100 to 100.
        max_tokens (Union[None, Unset, int]): The maximum number of tokens to generate in the chat completion.
        presence_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on whether they appear in the text so far.
        response_format (Union['ChatCompletionInputResponseFormatType0', None, Unset]): An object specifying the format
            that the model must output.
        seed (Union[None, Unset, int]): This feature is in Beta. If specified, our system will make a best effort to
            sample deterministically.
        stop (Union[None, Unset, str]): Up to 4 sequences where the API will stop generating further tokens.
        stream (Union[Unset, bool]): If set, partial message deltas will be sent, like in ChatGPT.
        temperature (Union[None, Unset, float]): What sampling temperature to use, between 0 and 2.
        top_p (Union[None, Unset, float]): An alternative to sampling with temperature, called nucleus sampling.
        tools (Union[Unset, List['ChatCompletionInputToolsItem']]): A list of tools the model may call. Currently, only
            functions are supported as a tool.
        user (Union[None, Unset, str]): A unique identifier representing your end-user.
    """

    project_id: int
    messages: List["Message"]
    session_id: Union[Unset, str] = UNSET
    repositories: Union[Unset, "Enhancement"] = UNSET
    model: Union[Unset, str] = UNSET
    system_prompt: Union[Unset, str] = UNSET
    frequency_penalty: Union[Unset, float] = UNSET
    logit_bias: Union["ChatCompletionInputLogitBiasType0", None, Unset] = UNSET
    max_tokens: Union[None, Unset, int] = UNSET
    presence_penalty: Union[Unset, float] = UNSET
    response_format: Union["ChatCompletionInputResponseFormatType0", None, Unset] = UNSET
    seed: Union[None, Unset, int] = UNSET
    stop: Union[None, Unset, str] = UNSET
    stream: Union[Unset, bool] = UNSET
    temperature: Union[None, Unset, float] = UNSET
    top_p: Union[None, Unset, float] = UNSET
    tools: Union[Unset, List["ChatCompletionInputToolsItem"]] = UNSET
    user: Union[None, Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_completion_input_logit_bias_type_0 import ChatCompletionInputLogitBiasType0
        from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0

        project_id = self.project_id

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        session_id = self.session_id

        repositories: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.repositories, Unset):
            repositories = self.repositories.to_dict()

        model = self.model

        system_prompt = self.system_prompt

        frequency_penalty = self.frequency_penalty

        logit_bias: Union[Dict[str, Any], None, Unset]
        if isinstance(self.logit_bias, Unset):
            logit_bias = UNSET
        elif isinstance(self.logit_bias, ChatCompletionInputLogitBiasType0):
            logit_bias = self.logit_bias.to_dict()
        else:
            logit_bias = self.logit_bias

        max_tokens: Union[None, Unset, int]
        if isinstance(self.max_tokens, Unset):
            max_tokens = UNSET
        else:
            max_tokens = self.max_tokens

        presence_penalty = self.presence_penalty

        response_format: Union[Dict[str, Any], None, Unset]
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ChatCompletionInputResponseFormatType0):
            response_format = self.response_format.to_dict()
        else:
            response_format = self.response_format

        seed: Union[None, Unset, int]
        if isinstance(self.seed, Unset):
            seed = UNSET
        else:
            seed = self.seed

        stop: Union[None, Unset, str]
        if isinstance(self.stop, Unset):
            stop = UNSET
        else:
            stop = self.stop

        stream = self.stream

        temperature: Union[None, Unset, float]
        if isinstance(self.temperature, Unset):
            temperature = UNSET
        else:
            temperature = self.temperature

        top_p: Union[None, Unset, float]
        if isinstance(self.top_p, Unset):
            top_p = UNSET
        else:
            top_p = self.top_p

        tools: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tools, Unset):
            tools = []
            for tools_item_data in self.tools:
                tools_item = tools_item_data.to_dict()
                tools.append(tools_item)

        user: Union[None, Unset, str]
        if isinstance(self.user, Unset):
            user = UNSET
        else:
            user = self.user

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "messages": messages,
            }
        )
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if repositories is not UNSET:
            field_dict["repositories"] = repositories
        if model is not UNSET:
            field_dict["model"] = model
        if system_prompt is not UNSET:
            field_dict["system_prompt"] = system_prompt
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if logit_bias is not UNSET:
            field_dict["logit_bias"] = logit_bias
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if seed is not UNSET:
            field_dict["seed"] = seed
        if stop is not UNSET:
            field_dict["stop"] = stop
        if stream is not UNSET:
            field_dict["stream"] = stream
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if tools is not UNSET:
            field_dict["tools"] = tools
        if user is not UNSET:
            field_dict["user"] = user

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        _temp_messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            _temp_messages.append(messages_item)
        messages = (None, json.dumps(_temp_messages).encode(), "application/json")

        session_id = (
            self.session_id
            if isinstance(self.session_id, Unset)
            else (None, str(self.session_id).encode(), "text/plain")
        )

        repositories: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.repositories, Unset):
            repositories = (None, json.dumps(self.repositories.to_dict()).encode(), "application/json")

        model = self.model if isinstance(self.model, Unset) else (None, str(self.model).encode(), "text/plain")

        system_prompt = (
            self.system_prompt
            if isinstance(self.system_prompt, Unset)
            else (None, str(self.system_prompt).encode(), "text/plain")
        )

        frequency_penalty = (
            self.frequency_penalty
            if isinstance(self.frequency_penalty, Unset)
            else (None, str(self.frequency_penalty).encode(), "text/plain")
        )

        logit_bias: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.logit_bias, Unset):
            logit_bias = UNSET
        elif isinstance(self.logit_bias, ChatCompletionInputLogitBiasType0):
            logit_bias = (None, json.dumps(self.logit_bias.to_dict()).encode(), "application/json")
        else:
            logit_bias = self.logit_bias

        max_tokens: Union[None, Unset, int]
        if isinstance(self.max_tokens, Unset):
            max_tokens = UNSET
        else:
            max_tokens = self.max_tokens

        presence_penalty = (
            self.presence_penalty
            if isinstance(self.presence_penalty, Unset)
            else (None, str(self.presence_penalty).encode(), "text/plain")
        )

        response_format: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ChatCompletionInputResponseFormatType0):
            response_format = (None, json.dumps(self.response_format.to_dict()).encode(), "application/json")
        else:
            response_format = self.response_format

        seed: Union[None, Unset, int]
        if isinstance(self.seed, Unset):
            seed = UNSET
        else:
            seed = self.seed

        stop: Union[None, Unset, str]
        if isinstance(self.stop, Unset):
            stop = UNSET
        else:
            stop = self.stop

        stream = self.stream if isinstance(self.stream, Unset) else (None, str(self.stream).encode(), "text/plain")

        temperature: Union[None, Unset, float]
        if isinstance(self.temperature, Unset):
            temperature = UNSET
        else:
            temperature = self.temperature

        top_p: Union[None, Unset, float]
        if isinstance(self.top_p, Unset):
            top_p = UNSET
        else:
            top_p = self.top_p

        tools: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.tools, Unset):
            _temp_tools = []
            for tools_item_data in self.tools:
                tools_item = tools_item_data.to_dict()
                _temp_tools.append(tools_item)
            tools = (None, json.dumps(_temp_tools).encode(), "application/json")

        user: Union[None, Unset, str]
        if isinstance(self.user, Unset):
            user = UNSET
        else:
            user = self.user

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
                "messages": messages,
            }
        )
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if repositories is not UNSET:
            field_dict["repositories"] = repositories
        if model is not UNSET:
            field_dict["model"] = model
        if system_prompt is not UNSET:
            field_dict["system_prompt"] = system_prompt
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if logit_bias is not UNSET:
            field_dict["logit_bias"] = logit_bias
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if seed is not UNSET:
            field_dict["seed"] = seed
        if stop is not UNSET:
            field_dict["stop"] = stop
        if stream is not UNSET:
            field_dict["stream"] = stream
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if tools is not UNSET:
            field_dict["tools"] = tools
        if user is not UNSET:
            field_dict["user"] = user

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_completion_input_logit_bias_type_0 import ChatCompletionInputLogitBiasType0
        from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0
        from ..models.chat_completion_input_tools_item import ChatCompletionInputToolsItem
        from ..models.enhancement import Enhancement
        from ..models.message import Message

        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = Message.from_dict(messages_item_data)

            messages.append(messages_item)

        session_id = d.pop("session_id", UNSET)

        _repositories = d.pop("repositories", UNSET)
        repositories: Union[Unset, Enhancement]
        if isinstance(_repositories, Unset):
            repositories = UNSET
        else:
            repositories = Enhancement.from_dict(_repositories)

        model = d.pop("model", UNSET)

        system_prompt = d.pop("system_prompt", UNSET)

        frequency_penalty = d.pop("frequency_penalty", UNSET)

        def _parse_logit_bias(data: object) -> Union["ChatCompletionInputLogitBiasType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logit_bias_type_0 = ChatCompletionInputLogitBiasType0.from_dict(data)

                return logit_bias_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ChatCompletionInputLogitBiasType0", None, Unset], data)

        logit_bias = _parse_logit_bias(d.pop("logit_bias", UNSET))

        def _parse_max_tokens(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_tokens = _parse_max_tokens(d.pop("max_tokens", UNSET))

        presence_penalty = d.pop("presence_penalty", UNSET)

        def _parse_response_format(data: object) -> Union["ChatCompletionInputResponseFormatType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_0 = ChatCompletionInputResponseFormatType0.from_dict(data)

                return response_format_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ChatCompletionInputResponseFormatType0", None, Unset], data)

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        def _parse_seed(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        seed = _parse_seed(d.pop("seed", UNSET))

        def _parse_stop(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        stop = _parse_stop(d.pop("stop", UNSET))

        stream = d.pop("stream", UNSET)

        def _parse_temperature(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        temperature = _parse_temperature(d.pop("temperature", UNSET))

        def _parse_top_p(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        top_p = _parse_top_p(d.pop("top_p", UNSET))

        tools = []
        _tools = d.pop("tools", UNSET)
        for tools_item_data in _tools or []:
            tools_item = ChatCompletionInputToolsItem.from_dict(tools_item_data)

            tools.append(tools_item)

        def _parse_user(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        user = _parse_user(d.pop("user", UNSET))

        chat_completion_input = cls(
            project_id=project_id,
            messages=messages,
            session_id=session_id,
            repositories=repositories,
            model=model,
            system_prompt=system_prompt,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            max_tokens=max_tokens,
            presence_penalty=presence_penalty,
            response_format=response_format,
            seed=seed,
            stop=stop,
            stream=stream,
            temperature=temperature,
            top_p=top_p,
            tools=tools,
            user=user,
        )

        chat_completion_input.additional_properties = d
        return chat_completion_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProviderInternalServerErrorCodeEnum(str, Enum):
    PROVIDERINTERNALSERVERERROR = "ProviderInternalServerError"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="InputDataPoint")


class InputDataPointDict(TypedDict):
    positive: bool
    project: int
    input_: NotRequired[Union[None, Unset, str]]
    output: NotRequired[Union[None, Unset, str]]
    trace: NotRequired[Union[None, Unset, str]]
    pass


@_attrs_define
class InputDataPoint:
    """
    Attributes:
        positive (bool):
        project (int):
        input_ (Union[None, Unset, str]):
        output (Union[None, Unset, str]):
        trace (Union[None, Unset, str]):
    """

    positive: bool
    project: int
    input_: Union[None, Unset, str] = UNSET
    output: Union[None, Unset, str] = UNSET
    trace: Union[None, Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        positive = self.positive

        project = self.project

        input_: Union[None, Unset, str]
        if isinstance(self.input_, Unset):
            input_ = UNSET
        else:
            input_ = self.input_

        output: Union[None, Unset, str]
        if isinstance(self.output, Unset):
            output = UNSET
        else:
            output = self.output

        trace: Union[None, Unset, str]
        if isinstance(self.trace, Unset):
            trace = UNSET
        else:
            trace = self.trace

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "positive": positive,
                "project": project,
            }
        )
        if input_ is not UNSET:
            field_dict["input"] = input_
        if output is not UNSET:
            field_dict["output"] = output
        if trace is not UNSET:
            field_dict["trace"] = trace

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        positive = (
            self.positive if isinstance(self.positive, Unset) else (None, str(self.positive).encode(), "text/plain")
        )

        project = self.project if isinstance(self.project, Unset) else (None, str(self.project).encode(), "text/plain")

        input_: Union[None, Unset, str]
        if isinstance(self.input_, Unset):
            input_ = UNSET
        else:
            input_ = self.input_

        output: Union[None, Unset, str]
        if isinstance(self.output, Unset):
            output = UNSET
        else:
            output = self.output

        trace: Union[None, Unset, str]
        if isinstance(self.trace, Unset):
            trace = UNSET
        else:
            trace = self.trace

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "positive": positive,
                "project": project,
            }
        )
        if input_ is not UNSET:
            field_dict["input"] = input_
        if output is not UNSET:
            field_dict["output"] = output
        if trace is not UNSET:
            field_dict["trace"] = trace

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        positive = d.pop("positive")

        project = d.pop("project")

        def _parse_input_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_output(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        output = _parse_output(d.pop("output", UNSET))

        def _parse_trace(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        trace = _parse_trace(d.pop("trace", UNSET))

        input_data_point = cls(
            positive=positive,
            project=project,
            input_=input_,
            output=output,
            trace=trace,
        )

        input_data_point.additional_properties = d
        return input_data_point

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
